<?php

namespace Drupal\commerce_affirm\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_affirm\Event\AffirmEvents;
use Drupal\commerce_affirm\Event\AffirmLocaleEvent;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\commerce_price\Price;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsRefundsInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsVoidsInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsAuthorizationsInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;

/**
 * Provides the Affirm Redirect payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "affirm_redirect",
 *   label = @Translation("Affirm (Redirect)"),
 *   display_label = @Translation("Affirm"),
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_affirm\PluginForm\RedirectForm",
 *   },
 *   requires_billing_information = FALSE,
 * )
 */
class Redirect extends OffsitePaymentGatewayBase implements SupportsRefundsInterface, SupportsVoidsInterface, SupportsAuthorizationsInterface {

  /**
   * The logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    $instance->logger = $container->get('logger.channel.commerce_affirm');
    $instance->httpClient = $container->get('http_client');
    $instance->eventDispatcher = $container->get('event_dispatcher');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $return = [
      'window_mode' => 'redirect',
      'public_key' => '',
      'private_key' => '',
      'financial_key' => '',
      'default_locale' => 'en_US',
      'country_code' => 'USA',
      'log' => FALSE,
    ];
    return $return + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['window_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Window mode'),
      '#options' => [
        'redirect' => $this->t('Redirect'),
        'modal' => $this->t('Modal'),
      ],
      '#default_value' => $this->configuration['window_mode'],
    ];
    // Public key.
    $form['public_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Public Key'),
      '#description' => $this->t('Add your public key for the gateway.'),
      '#default_value' => $this->configuration['public_key'],
      '#required' => TRUE,
    ];
    // Private key.
    $form['private_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Private Key'),
      '#description' => $this->t('Add your private key for the gateway.'),
      '#default_value' => $this->configuration['private_key'],
      '#required' => TRUE,
    ];
    // Financial Product Key.
    $form['financial_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Financial Product Key'),
      '#description' => $this->t('Add your financial key for the gateway.'),
      '#default_value' => $this->configuration['financial_key'],
      '#required' => TRUE,
    ];
    // Default locale.
    $form['default_locale'] = [
      '#type' => 'select',
      '#options' => [
        'en_US' => $this->t('USA - English'),
        'en_CA' => $this->t('Canada - English'),
        'fr_CA' => $this->t('Canada - French'),
      ],
      '#title' => $this->t('Default locale'),
      '#default_value' => $this->configuration['default_locale'],
    ];
    // Country code.
    $form['country_code'] = [
      '#type' => 'select',
      '#options' => [
        'USA' => $this->t('USA'),
        'CAN' => $this->t('Canada'),
      ],
      '#title' => $this->t('Country code'),
      '#default_value' => $this->configuration['country_code'],
    ];
    $form['log'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Log API and related data for debugging'),
      '#default_value' => $this->configuration['log'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['window_mode'] = $values['window_mode'];
      $this->configuration['public_key'] = $values['public_key'];
      $this->configuration['private_key'] = $values['private_key'];
      $this->configuration['financial_key'] = $values['financial_key'];
      $this->configuration['default_locale'] = $values['default_locale'];
      $this->configuration['country_code'] = $values['country_code'];
      $this->configuration['log'] = $values['log'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $data = [
      'transaction_id' => $request->request->get('checkout_token'),
      'order_id' => $order->id(),
    ];
    try {
      $response = $this->apiRequest('authorization', $data);
    }
    catch (\Exception $e) {
      $this->logger->warning($e->getMessage());
      throw new PaymentGatewayException($this->t('We could not complete your payment with Affirm. Please try again or contact us if the problem persists.'));
    }

    if (empty($response)) {
      $this->messenger()->addError($this->t('We could not complete your payment with Affirm. Please try again or contact us if the problem persists.'));
      throw new PaymentGatewayException($this->t('We could not complete your payment with Affirm. Please try again or contact us if the problem persists.'));
    }

    if ($response->getStatusCode() == 200) {
      $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
      $response_json = json_decode($response->getBody());
      // Checks if the response contains an error.
      if (isset($response_json->status_code)) {
        // Display an error message and remain on the same page.
        $this->messenger()->addError($this->t('We could not complete your payment with Affirm. Please try again or contact us if the problem persists.'));
        throw new PaymentGatewayException($this->t('We could not complete your payment with Affirm. Please try again or contact us if the problem persists.'));
      }

      /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
      $payment = $payment_storage->create([
        'state' => 'authorization',
        'amount' => $order->getTotalPrice(),
        'payment_gateway' => $this->parentEntity->id(),
        'order_id' => $order->id(),
        'remote_id' => $response_json->id,
        'remote_status' => 'authorize',
      ]);
      $payment->save();
      /** @var \Drupal\commerce_checkout\Entity\CheckoutFlowInterface $checkout_flow */
      $checkout_flow = $order->get('checkout_flow')->entity;
      $capture = $checkout_flow->getPlugin()->getConfiguration()['panes']['payment_process']['capture'];
      if ($capture) {
        $this->capturePayment($payment);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function capturePayment(PaymentInterface $payment, Price $amount = NULL) {
    if (!empty($amount) && $amount->lessThan($payment->getAmount())) {
      $this->logger->warning($this->t('Partial capture attempted for payment @id.', ['@id' => $payment->id()]));
      throw new PaymentGatewayException($this->t('Affirm does not support partial captures.'));
    }
    $amount = $payment->getAmount();

    $data = [
      'amount' => $this->minorUnitsConverter->toMinorUnits($amount),
    ];
    try {
      $response = $this->apiRequest('capture', $data, $payment);
    }
    catch (\Exception $e) {
      $this->logger->warning($e->getMessage());
      throw new PaymentGatewayException($this->t('We could not complete your payment with Affirm. Please try again or contact us if the problem persists.'));
    }
    $response_json = json_decode($response->getBody());

    if (empty($response_json) || isset($response_json->status_code)) {
      throw new PaymentGatewayException($this->t('We could not complete your payment with Affirm. Please try again or contact us if the problem persists.'));
    }
    $payment->setState('completed');
    $payment->setRemoteState($response_json->type);
    $payment->save();
  }

  /**
   * {@inheritdoc}
   */
  public function voidPayment(PaymentInterface $payment) {
    try {
      $response = $this->apiRequest('void', [], $payment);
    }
    catch (\Exception $e) {
      $this->logger->warning($e->getMessage());
      throw new PaymentGatewayException($this->t('We could not void your payment with Affirm. Please try again or contact us if the problem persists.'));
    }
    $response_json = json_decode($response->getBody());

    if (empty($response_json) || isset($response_json->status_code)) {
      throw new PaymentGatewayException($this->t('We could not void your payment with Affirm. Please try again or contact us if the problem persists.'));
    }
    $payment->setState('authorization_voided');
    $payment->setRemoteState($response_json->type);
    $payment->save();
  }

  /**
   * {@inheritdoc}
   */
  public function refundPayment(PaymentInterface $payment, Price $amount = NULL) {
    if (!$amount) {
      $amount = $payment->getAmount();
    }
    $data = [
      'amount' => $this->minorUnitsConverter->toMinorUnits($amount),
    ];
    try {
      $response = $this->apiRequest('refund', $data, $payment);
    }
    catch (\Exception $e) {
      $this->logger->warning($e->getMessage());
      throw new PaymentGatewayException($this->t('We could not refund your payment with Affirm. Please try again or contact us if the problem persists.'));
    }
    $response_json = json_decode($response->getBody());

    if (empty($response_json) || isset($response_json->status_code)) {
      throw new PaymentGatewayException($this->t('We could not refund your payment with Affirm. Please try again or contact us if the problem persists.'));
    }
    $old_refunded_amount = $payment->getRefundedAmount();
    $new_refunded_amount = $old_refunded_amount->add($amount);
    if ($new_refunded_amount->lessThan($payment->getAmount())) {
      $payment->setState('partially_refunded');
    }
    else {
      $payment->setState('refunded');
    }
    $payment->setRemoteState($response_json->type);
    $payment->setRefundedAmount($new_refunded_amount);
    $payment->save();
  }

  /**
   * Returns the base URL to Affirm API server.
   *
   * @return string
   *   The Base URL to use to submit requests to the Affirm API server.
   */
  public function serverUrl() {
    if ($this->getMode() === 'test') {
      return 'https://api.global-sandbox.affirm.com/api/v1/transactions';
    }
    else {
      return 'https://api.global.affirm.com/api/v1/transactions';
    }
  }

  /**
   * Returns the URL to the Affirm server determined by the transaction type.
   *
   * @param string $request_type
   *   The transaction request type (authorization/capture/refund/void).
   * @param \Drupal\commerce_payment\Entity\PaymentInterface|null $payment
   *   The payment, NULL if not set.
   *
   * @return string|bool
   *   The URL to use to submit requests to the Affirm server.
   */
  public function apiServerUrl(string $request_type, PaymentInterface $payment = NULL) {
    $api_url = $this->serverUrl();

    switch ($request_type) {
      case 'authorization':
        return $api_url;

      case 'capture':
      case 'refund':
      case 'void':
        if (empty($payment)) {
          $this->messenger()->addStatus($this->t('You can not create a @request_type request without specifying a payment.', [
            '@request_type' => $request_type]));
          return FALSE;
        }
        $transaction_id = $payment->getRemoteId();
        return "$api_url/$transaction_id/$request_type";
    }

    return FALSE;
  }

  /**
   * Perform a request against the Affirm API.
   *
   * @param string $request_type
   *   The transaction request type (authorization/capture/refund/void).
   * @param array $data
   *   Data to send to Affirm.
   * @param \Drupal\commerce_payment\Entity\PaymentInterface|null $payment
   *   The payment, NULL if not set.
   *
   * @return \Psr\Http\Message\ResponseInterface
   *   The Affirm response.
   */
  public function apiRequest(string $request_type, array $data, PaymentInterface $payment = NULL) {
    $options = [
      'headers' => [
        'Content-Type' => 'application/json',
        'country-code' => $this->getCountryCode(),
      ],
      'auth' => [
        $this->getPublicKey(),
        $this->getPrivateKey(),
      ],
    ];

    // A json parameter should not be added for a Void request.
    if ($request_type !== 'void') {
      $options['json'] = $data;
    }

    return $this->httpClient->post($this->apiServerUrl($request_type, $payment), $options);
  }

  /**
   * Gets the Affirm public key.
   *
   * @return string
   *   Returns the public key.
   */
  public function getPublicKey() {
    return $this->configuration['public_key'];
  }

  /**
   * Gets the Affirm private key.
   *
   * @return string
   *   Returns the private key.
   */
  public function getPrivateKey() {
    return $this->configuration['private_key'];
  }

  /**
   * Gets the Affirm financial key.
   *
   * @return string
   *   Returns the financial key.
   */
  public function getFinancialKey() {
    return $this->configuration['financial_key'];
  }

  /**
   * Gets the Affirm default locale.
   *
   * @return string
   *   Returns the default locale.
   */
  public function getDefaultLocale() {
    return $this->configuration['default_locale'];
  }

  /**
   * Gets the Affirm default locale altered by other modules.
   *
   * @return string
   *   Returns the default locale altered by other modules.
   */
  public function getLocale() {
    $locale = $this->getDefaultLocale();

    // Allow modules to alter the default locale.
    $event = new AffirmLocaleEvent($locale);
    $this->eventDispatcher->dispatch($event, AffirmEvents::AFFIRM_LOCALE);
    $locale = $event->getLocale();

    return $locale;
  }

  /**
   * Gets the Affirm country code.
   *
   * @return string
   *   Returns the country code.
   */
  public function getCountryCode() {
    return $this->configuration['country_code'];
  }

}
