<?php

namespace Drupal\commerce_affirm\PluginForm;

use Drupal\commerce_price\MinorUnitsConverterInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_price\Calculator;
use Drupal\commerce_promotion\Entity\Promotion;
use Drupal\commerce_order\Adjustment;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_affirm\Event\AffirmTransactionDataPreSend;
use Drupal\commerce_affirm\Event\AffirmEvents;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Drupal\Core\Messenger\MessengerTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

class RedirectForm extends PaymentOffsiteForm implements ContainerInjectionInterface {

  use MessengerTrait;

  /**
   * The minor units converter.
   *
   * @var \Drupal\commerce_price\MinorUnitsConverterInterface
   */
  protected $minorUnitsConverter;

  /**
   * Constructs a new RedirectForm object.
   *
   * @param \Drupal\commerce_price\MinorUnitsConverterInterface $minor_units_converter
   *   The minor units converter.
   */
  public function __construct(MinorUnitsConverterInterface $minor_units_converter) {
    $this->minorUnitsConverter = $minor_units_converter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('commerce_price.minor_units_converter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    /** @var \Drupal\commerce_affirm\Plugin\Commerce\PaymentGateway\Redirect $plugin */
    $plugin = $this->plugin;

    $configuration = $plugin->getConfiguration();

    // Return an error if the enabling action's settings haven't been configured.
    if (empty($plugin->getPublicKey()) || empty($plugin->getFinancialKey())) {
      $this->messenger->addError($this->t('Affirm is not configured for use. You must check your settings.'));
      return [];
    }
    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->getEntity();
    $order = $payment->getOrder();
    $order_total = $order->getTotalPrice();

    $extension_list = \Drupal::getContainer()->get('extension.list.module');
    $commerce_module_info = $extension_list->getExtensionInfo('commerce');
    $affirm_module_info = $extension_list->getExtensionInfo('commerce_affirm');
    if ($commerce_module_info['version'] == '') {
      $commerce_module_info['version'] = '8.x-1.??';
    }

    // Creates a data array to be passed onto the request.
    $data = [
      // General settings.
      'ApiMode'             => $plugin->getMode() === 'live' ? 'live' : 'sandbox',
      'PublicKey'           => $plugin->getPublicKey(),
      'Locale'              => $plugin->getLocale(),
      'CountryCode'         => $plugin->getCountryCode(),
      'FinancialProductKey' => $plugin->getFinancialKey(),
      'OrderId'             => $order->id(),
      'Email'               => $order->getEmail(),
      'CancelUrl'           => $form['#cancel_url'],
      'ConfirmUrl'          => $form['#return_url'],
      'items'               => [],
      'metadata'            => [
        'platform_type' => 'Drupal',
        'platform_version' => 'Drupal ' . \Drupal::VERSION . '; Drupal Commerce ' . $commerce_module_info['version'],
        'platform_affirm' => $affirm_module_info['version'],
        'shipping_type' => '',
        // This gets sent back by Affirm so that we can use it in onReturn().
        'capture' => $form['#capture'],
      ],
      'ShippingTotal'       => 0,
      'TaxAmount'           => 0,
      'ProductsTotal'       => $this->minorUnitsConverter->toMinorUnits($order_total),
      'OrderCurrency'       => $order_total->getCurrencyCode(),
    ];
    if ($configuration['window_mode'] == 'modal') {
      $data['metadata']['mode'] = 'modal';
    }

    // Adds information about the billing profile.
    if ($billing_profile = $order->getBillingProfile()) {
      /** @var \Drupal\address\Plugin\Field\FieldType\AddressItem $billing_address */
      $billing_address = $billing_profile->get('address')->first();
      $data += [
        'BillingFullName'        => $billing_address->getGivenName() . ' ' . $billing_address->getFamilyName(),
        'BillingFirstName'       => $billing_address->getGivenName(),
        'BillingLastName'        => $billing_address->getFamilyName(),
        'BillingAddressLn1'      => $billing_address->getAddressLine1(),
        'BillingAddressLn2'      => $billing_address->getAddressLine2(),
        'BillingAddressPostCode' => $billing_address->getPostalCode(),
        'BillingAddressState'    => $billing_address->getAdministrativeArea(),
        'BillingAddressCity'     => $billing_address->getLocality(),
        'BillingAddressCountry'  => $billing_address->getCountryCode(),
        'BillingTelephone'       => '',
      ];
    }

    // Use the shipping profile if the shipping module exists and the order
    // contain a shipping profile otherwise use the Billing profile.
    $order_profiles = $order->collectProfiles();
    $profile = $order_profiles['shipping'] ?? $billing_profile;
    if ($profile) {
      /** @var \Drupal\address\Plugin\Field\FieldType\AddressItem $shipping_address */
      $shipping_address = $profile->get('address')->first();
      $data += [
        'ShippingFullName'        => $shipping_address->getGivenName() . ' ' . $shipping_address->getFamilyName(),
        'ShippingFirstName'       => $shipping_address->getGivenName(),
        'ShippingLastName'        => $shipping_address->getFamilyName(),
        'ShippingAddressLn1'      => $shipping_address->getAddressLine1(),
        'ShippingAddressLn2'      => $shipping_address->getAddressLine2(),
        'ShippingAddressCountry'  => $shipping_address->getCountryCode(),
        'ShippingAddressPostCode' => $shipping_address->getPostalCode(),
        'ShippingAddressState'    => $shipping_address->getAdministrativeArea(),
        'ShippingAddressCity'     => $shipping_address->getLocality(),
        'ShippingTelephone'       => '',
      ];
    }

    foreach ($order->getItems() as $order_item) {
      $data['items'][] = [
        'sku' => $order_item->getPurchasedEntity()->getSku(),
        'qty' => $order_item->getQuantity(),
        'display_name' => $order_item->getTitle(),
        'item_url' => $order_item->getPurchasedEntity()->toUrl()->setAbsolute()->toString(),
        'item_image_url' => '',
        'unit_price' => $this->minorUnitsConverter->toMinorUnits($order_item->getUnitPrice()),
      ];
      foreach ($order_item->getAdjustments() as $adjustment) {
        $this->handleAdjustment($order, $adjustment, $data);
      }
    }

    foreach ($order->getAdjustments() as $adjustment) {
      $this->handleAdjustment($order, $adjustment, $data);
    }
    sort($data['items']);

    // Allow modules to alter the order data sent to Affirm before the request.
    /** @var \Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher $event_dispatcher */
    $event_dispatcher = \Drupal::service('event_dispatcher');
    $event = new AffirmTransactionDataPreSend($data, $order);
    $event_dispatcher->dispatch($event,AffirmEvents::AFFIRM_TRANSACTION_DATA_PRESEND);

    ksort($data);
    $form['#attached']['library'][] = 'commerce_affirm/affirm-checkout';

    // Pass checkout settings through to front-end.
    $form['#attached']['drupalSettings']['commerce_affirm'] = $data;

    if ($configuration['log']) {
      \Drupal::logger('commerce_affirm')->debug('@function debug data: <pre>@data</pre>', [
        '@function' => __FUNCTION__,
        '@data' => print_r($data, TRUE),
      ]);
    }

    return $form;
  }

  /**
   * Turn adjustment data into the corresponding checkout object data.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   The order.
   * @param \Drupal\commerce_order\Adjustment $adjustment
   *   The adjustment.
   * @param array $data
   *   The Affirm checkout object data as an array.
   */
  protected function handleAdjustment(OrderInterface $order, Adjustment $adjustment, array &$data) {
    switch ($adjustment->getType()) {
      case 'promotion':
        $promotion = Promotion::load($adjustment->getSourceId());
        if ($promotion->hasCoupons()) {
          /** @var \Drupal\commerce_promotion\Entity\Coupon $coupon */
          foreach ($order->coupons->referencedEntities() as $coupon) {
            if ($coupon->getPromotionId() == $promotion->id()) {
              if (!isset($data['discounts'][$coupon->getCode()])) {
                $data['discounts'][$coupon->getCode()] = [
                  'discount_display_name' => $promotion->getName(),
                  'discount_amount' => '0',
                ];
                break;
              }
            }
          }
          $data['discounts'][$coupon->getCode()]['discount_amount'] = Calculator::add($data['discounts'][$coupon->getCode()]['discount_amount'], Calculator::multiply(-1, $this->minorUnitsConverter->toMinorUnits($adjustment->getAmount())));
        }
        else {
          if (!isset($data['discounts'][$promotion->id()])) {
            $data['discounts'][$promotion->id()] = [
              'discount_display_name' => $promotion->getName(),
              'discount_amount' => '0',
            ];
          }
          $data['discounts'][$promotion->id()]['discount_amount'] = Calculator::add($data['discounts'][$promotion->id()]['discount_amount'], Calculator::multiply(-1, $this->minorUnitsConverter->toMinorUnits($adjustment->getAmount())));
        }
        break;

      case 'shipping':
        if (!isset($data['ShippingTotal'])) {
          $data['ShippingTotal'] = '0';
        }
        $data['ShippingTotal'] = Calculator::add($data['ShippingTotal'], $this->minorUnitsConverter->toMinorUnits($adjustment->getAmount()));
        break;

      case 'tax':
        if (!isset($data['TaxAmount'])) {
          $data['TaxAmount'] = 0;
        }
        $data['TaxAmount'] = Calculator::add($data['TaxAmount'], $this->minorUnitsConverter->toMinorUnits($adjustment->getAmount()));
        break;
    }
  }

}
